#
# This File contains the Exceptions related to PASTA-MAN only.
#

class InvalidKeyword(Exception):
    pass

class InvalidExportType(Exception):
    pass

class NoneTypeVariable(Exception):
    pass

class OptError(Exception):
    pass